#!/bin/bash -e

export LD_PRELOAD="/usr/local/lib/libthrift.so $WORKSPACE/bin/libgenthrift.so $WORKSPACE/bin/libcassandra.so"

LOCAL_IP=${LOCAL_IP:=127.0.0.1}
LOCAL_PORT=`$WORKSPACE/tests/helpers/freePort.sh`
JMX_PORT=`$WORKSPACE/tests/helpers/freePort.sh`
STORAGE_PORT=`$WORKSPACE/tests/helpers/freePort.sh`
CQL_PORT=`$WORKSPACE/tests/helpers/freePort.sh`
CASSANDRA_VERSION=${CASSANDRA_VERSION:=1.2.19}	# use value of CASSANDRA_VERSION or 1.2.19
LOGS_DIR=${LOGS_DIR:=$WORKSPACE/tmp/}			# so Jenkins can archive some Cassandra logs

filesDirectory="$WORKSPACE/tmp/"
mkdir -p $filesDirectory

cassandraName="apache-cassandra-$CASSANDRA_VERSION"
cassandraInstaller="$filesDirectory/$cassandraName-bin.tar.gz"
cd $filesDirectory
if [ ! -e "$cassandraInstaller" ]; then
	wget http://packages.inconcert/repo/$cassandraName-bin.tar.gz
fi

if [ ! -e "$cassandraInstaller" ]; then
	echo "Cassandra installer not found"
	return 1
fi

cd $filesDirectory
setupDir=`mktemp -d`
cd $setupDir
echo "Uncompressing Cassandra installer in temp dir. $setupDir" > /dev/stderr
tar -xzf $cassandraInstaller 1>/dev/null 2>/dev/null
cassandraDirectory="$setupDir/$cassandraName"

if [ ! -e "$cassandraDirectory/conf/cassandra.yaml" ]; then
	echo "Cassandra configuration file not found"
	rm -fR $cassandraDirectory
	return 1
fi

ln -s $cassandraDirectory/lib/$cassandraName.jar $cassandraDirectory/lib/apache-cassandra.jar

# Change YAML in order to setup a local environment
escapedDir=`echo "$cassandraDirectory" | sed -e 's/\//\\\\\//g'`
escapedDir=`echo "$escapedDir" | sed -e 's/\./\\\./g'`
sed -i -e  "s/rpc_address:.*/rpc_address: $LOCAL_IP/"  "$cassandraDirectory/conf/cassandra.yaml"
sed -i -e  "s/listen_address:.*/listen_address: $LOCAL_IP/"  "$cassandraDirectory/conf/cassandra.yaml"
sed -i -e  "s/rpc_port:.*/rpc_port: $LOCAL_PORT/"  "$cassandraDirectory/conf/cassandra.yaml"
sed -i -e  "s/storage_port:.*/storage_port: $STORAGE_PORT/"  "$cassandraDirectory/conf/cassandra.yaml"
sed -i -e  "s/native_transport_port:.*/native_transport_port: $CQL_PORT/"  "$cassandraDirectory/conf/cassandra.yaml"
sed -i -e  "s/seeds: \"127.0.0.1\"/seeds: \"$LOCAL_IP\"/" "$cassandraDirectory/conf/cassandra.yaml"

sed -i -e  "s/\/var\/lib\/cassandra\/data/$escapedDir\/data/"  "$cassandraDirectory/conf/cassandra.yaml"
sed -i -e  "s/\/var\/lib\/cassandra\/commitlog/$escapedDir\/commitlog/"  "$cassandraDirectory/conf/cassandra.yaml"
sed -i -e  "s/\/var\/lib\/cassandra\/saved_caches/$escapedDir\/saved_caches/"  "$cassandraDirectory/conf/cassandra.yaml"
sed -i -e  "s/\/var\/log\/cassandra\/system\.log/$escapedDir\/system\.log/" "$cassandraDirectory/conf/log4j-server.properties"
sed -i -e  "s/^JMX_PORT=.*/JMX_PORT=\"$JMX_PORT\"/g" "$cassandraDirectory/conf/cassandra-env.sh"


# Environment variables for everything to go on wheels on this session
export CASSANDRA_HOME=$cassandraDirectory

echo "" > $cassandraDirectory/conf/cassandra.in.sh

CASSANDRA_INCLUDE=$cassandrDirectory/conf/cassandra.in.sh
CASSANDRA_CONF=$cassandraDirectory/conf
CLASSPATH=$cassandraDirectory/conf
for jar in $cassandraDirectory/lib/*.jar; do
	CLASSPATH=$CLASSPATH:$jar
done

export CASSANDRA_INCLUDE=$CASSANDRA_INCLUDE
export CASSANDRA_CONF=$CASSANDRA_CONF
export CLASSPATH=$CLASSPATH

# The first existing directory is used for JAVA_HOME
JDK_DIRS="/usr/lib/jvm/java-6-sun /usr/lib/jvm/java-6-openjdk"
for jdir in $JDK_DIRS; do
	if [ -r "$jdir/bin/java" -a -z "${JAVA_HOME}" ]; then
 		export JAVA_HOME="$jdir"
	fi
done


# Start cassandra
echo "Starting $cassandraDirectory/bin/cassandra on port $LOCAL_PORT"
$cassandraDirectory/bin/cassandra  | tee -a $LOGS_DIR/cassandra_logfile $cassandraDirectory/logfile
sleep 20
cassandraPid=`ps ux | grep -v "teardown" | grep $cassandraDirectory | awk '/cassandra/ && !/awk/ {print $2}'`
if [ ! "$cassandraPid" ]; then
	echo "Cassandra did not start on the temporary location"
	rm -fR $cassandraDirectory
	return 1
fi


# Wait up to 2 minutes for everything to be alright
cassandraListening=0
for i in {1..120}
do
	sleep 1
# Change the isListening line when using plain Cassandra
#	isListening=`grep "Listening for thrift clients" $tmpOutputFile`
	isListening=`grep "state jump to normal" $cassandraDirectory/logfile`
	if [ -n "$isListening" ]; then
		cassandraListening=1
		break;
	fi
done

if [ $cassandraListening -eq 0 ]; then
	echo "Cassandra is not listening to clients after 2 minutes" > /dev/stderr
	rm -fR $cassandraDirectory
	return 1
fi

echo "Creating Cassandra schema" > /dev/stderr
$cassandraDirectory/bin/cassandra-cli -h $LOCAL_IP -p $LOCAL_PORT -f $WORKSPACE/sources/agent/schemas/cassandra/schema.cassandra > /dev/stderr
if [ $? -ne 0 ]; then
	echo "Failed to create schema." > /dev/stderr
	rm -fR $cassandraDirectory
	return 1
fi

echo "Schema created, creating indexing schema" > /dev/stderr
lua -e 'package.path = package.path .. ";../src/?.lua"
	local index = require "i6index.index"
	local schema = require "i6index.schema"
	local cassandra = require "lua_cassandra"

	cassandra.seed("'$LOCAL_IP'", '$LOCAL_PORT')

	index.drop(schema)
	index.create(schema)'

if [ $? -ne 0 ]; then
	echo "Failed to index." > /dev/stderr
	rm -fR $cassandraDirectory
	return 1
fi

echo "Cassandra is set up, loading initial data." > /dev/stderr

# Load cassandra initial data
cd $WORKSPACE/sources/agent/helpers/
lua initialdata4cassandra.lua $LOCAL_IP $LOCAL_PORT $WORKSPACE/sources/agent/helpers/spamfilterdb/
cd -

echo "CassandraPort:"$LOCAL_PORT " Path:"$cassandraDirectory " PID:"$cassandraPid

export CASSANDRA_PATH=$cassandraDirectory
export CASSANDRA_PORT=$LOCAL_PORT
export CASSANDRA_PID=$cassandraPid
